#############################################################
#
#   Program: table1.R
#   Project: IeDEA TB project
#   PI:  Muktar Aliyu
#   Statistican/Programmer: Meridith Blevins, MS
#   Purpose: analyze Nigeria R01 study characteristics
#   Notes:
#   Created: 10 April 2015
#   Revisions:
#
#
#############################################################
library(rms)

# if(!exists('new_out')){
#    new_out <- 'recfail.factor'
#    new_out_short <- 'recfail'
# }

if(art_lvls == 4){
   lvl_art <- c('On ART',
                paste('Initiated within ',post_tbdx_time/7,' weeks',sep=''),
                paste('Initiated after ',post_tbdx_time/7,' weeks',sep=''),
               'No ART during TB treatment')
   art_lbl <- c(paste('ART in ',post_tbdx_time/7,' weeks vs. None',sep=''),
                paste('ART after ',post_tbdx_time/7,' weeks vs. None',sep=''))
} else if (art_lvls == 3){
   lvl_art <- levels(relevel(d1$art_status_overall.factor, ref=art_ref))
   art_lbl <- c(paste(lvl_art[2],' vs. ',lvl_art[1],sep=''),
                paste(lvl_art[3],' vs. ',lvl_art[1],sep=''))
}

if(post_tbdx_time/7 == 2){
# if(!is.null(new_art)){
#    lvl_art <- c('On ART','Initiated within 2 weeks','Initiated after 2 weeks','No ART during TB treatment')
#    art_lbl <- c('ART in 2 weeks vs. None','ART after 2 weeks vs. None')
#    rcsval_cd4 <- c(10,30)
   cd4_lbl <- 'CD4+ cell count (30 vs. 10 cells)'
} else if(post_tbdx_time/7 == 8){
#    lvl_art <- c("On ART","Initiated within 8 weeks","Initiated after 8 weeks","No ART during TB treatment")
#    art_lbl <- c('ART in 8 weeks vs. None','ART after 8 weeks vs. None')
#    rcsval_cd4 <- c(50,200)
   cd4_lbl <- 'CD4+ cell count (200 vs. 50 cells)'
}


## female
d1$female <- as.numeric(d1$X5...Sex=="Female")
## smear positive
d1$smearpos <- as.numeric(d1$AFB.smear=="Positive")
## culture positive
d1$culturepos <- as.numeric(d1$AFB.Culture.=="Positive")
## bacteriologically confirmed
d1$baccon <- as.numeric(d1$microtest == "At least 1 positive result")
## extrapulmonary
d1$extrapulm <- as.numeric(d1$X16...Site.of.disease == "Extra-pulmonary")
## directly observed therapy
d1$dot.partial <- as.numeric(d1$X19...Was.treatment.directly.observed..DOT..=="Yes, part of treatment")
d1$dot.complete <- as.numeric(d1$X19...Was.treatment.directly.observed..DOT..=="Yes entire treatment")
## art status
d1$art1 <- as.numeric(d1$art_status == "Active at TB Diagnosis")
d1$art2 <- as.numeric(d1$art_status == paste("Initiated/Restarted within ",post_tbdx_time/7," weeks of TB Treatment Start",sep=''))
d1$art3 <- as.numeric(d1$art_status == paste("Initiated/Restarted ",post_tbdx_time/7," weeks after of TB Treatment Start",sep=''))



# df1 <- with(d1,data.frame(patient,age=age.tbdx,female,
#                    bmi1=BMI.at.start.of.TB.treatment,weight1=X10...Weight.at.start.of.TB.treatment.value,
#                    smearpos,culturepos,baccon,extrapulm,dot.partial,dot.complete,cd4=X25...CD4.cell.count.value,
#                    art1,art2,art3,bmi2=BMI.at.end.of.TB.treatment,weight2=X11...Weight.at.end.of.TB.treatment.value))
df1 <- with(d1,data.frame(patient,
                          age=age.tbdx,
                          female,
                          bmi1=BMI.at.start.of.TB.treatment,
                          weight1=X10...Weight.at.start.of.TB.treatment.value,
                          smearpos,
                          culturepos,
                          baccon,
                          extrapulm,
                          dot.partial,
                          dot.complete,
                          cd4=X25...CD4.cell.count.value,
                          art1,
                          art2,
                          art3,
                          art_status,
                          art_status_postdx,
                          art_status_overall.factor,
                          art_bf_xwks,
                          art_within_xwks,
                          art_gt_xwks,
                          bmi2=BMI.at.end.of.TB.treatment,
                          weight2=X11...Weight.at.end.of.TB.treatment.value,
                          region=as.factor(region),
                          X2...Cohort.or.Center.ID,
#                          dot_intensive,
#                           dot_intensive.factor,
                          dot_continuation,
#                           dot_continuation.factor,
                          year_tbdx_cat.factor))
df1[[new_out_short]] <- as.numeric(d1[[new_out_short]])

ai_vars <- ifelse(art_lvls == 4,paste(c('art1','art2','art3'),collapse='+'),
                  ifelse(art_lvls == 3,paste(c('art_bf_xwks','art_within_xwks','art_gt_xwks'),collapse='+'),NA))
# Added vars was 'year_tbdx_cat.factor' and 'region' until 9/5/2018; After this date, removing region or site from the imputation model
# since the DOT variable used was no longer the individual-level ones of dot.partial and dot.complete.  Rather, we used the site-level ones
# that I created of dot_intensive and dot_continuation.
added_vars <- c('year_tbdx_cat.factor','region') # Had previously imputed with region but adjusted for site; no longer
aI_fmla <- as.formula(paste('~ ',new_out_short,' +age+female+bmi1+weight1+
                               smearpos+culturepos+baccon+extrapulm+
#                               dot_intensive+
                               dot_continuation+cd4+',
                               ai_vars,'+bmi2+weight2+',paste(added_vars, collapse='+'),sep='')) # added added_vars 3/8/18 -- April wanted all models adjusted for region and year of TB dx; changed region to X2...Cohort.or.Center.ID
#                                smearpos+culturepos+baccon+extrapulm+dot.partial+dot.complete+cd4+',

# a1 <- aregImpute(~ age+female+bmi1+weight1+
#                  smearpos+culturepos+baccon+extrapulm+dot.partial+dot.complete+cd4+
#                  art1+art2+art3+bmi2+weight2, n.impute=1,data=df1,nk=4)
set.seed(1)
a1 <- aregImpute(aI_fmla, n.impute=1,data=df1,nk=4)

imputed <- impute.transcan(a1, imputation=1, data=df1, list.out=TRUE,pr=FALSE, check=FALSE)

# # Replacing region with X2...Cohort.or.Center.ID in attributes of a1:
# aI_fmla_wSite <- as.formula(paste('~ age+female+bmi1+weight1+
#                                      smearpos+culturepos+baccon+extrapulm+dot.partial+dot.complete+cd4+',
#                                      ai_vars,'+bmi2+weight2+year_tbdx_cat.factor+X2...Cohort.or.Center.ID',sep='')) # added added_vars 3/8/18 -- April wanted all models adjusted for region and year of TB dx; changed region to X2...Cohort.or.Center.ID
# a1[['formula']] <- aI_fmla_wSite
# names(a1[['na']])[which(names(a1[['na']]) == 'region')] <- 'X2...Cohort.or.Center.ID'
# names(a1[['nna']])[which(names(a1[['nna']]) == 'region')] <- 'X2...Cohort.or.Center.ID'
# names(a1[['type']])[which(names(a1[['type']]) == 'region')] <- 'X2...Cohort.or.Center.ID'
# a1[['cat.levels']][which(names(a1[['cat.levels']]) == 'region')][[1]] <- c(levels(df1$X2...Cohort.or.Center.ID))
# names(a1[['cat.levels']])[which(names(a1[['cat.levels']]) == 'region')] <- 'X2...Cohort.or.Center.ID'
# a1[['df']][which(names(a1[['df']]) == 'region')] <- length(levels(df1$X2...Cohort.or.Center.ID)) - 1
# names(a1[['df']])[which(names(a1[['df']]) == 'region')] <- 'X2...Cohort.or.Center.ID'

# No longer needed since not adjusting for site or region:
# which_region <- which(names(imputed) == 'region')
# names(imputed)[which_region] <- 'X2...Cohort.or.Center.ID'
# imputed[[which_region]] <- df1$X2...Cohort.or.Center.ID

idata <- df1
idata[names(imputed)] <- imputed
names(idata)

# idata_new <- df1
# idata_new[names(imputed_new)] <- imputed_new
# names(idata_new)
# 11/19/18:  Removing dot_intensive from all analyses:
getpca <- function(var=c('age','female','bmi1','weight1','smearpos','culturepos','baccon','extrapulm','dot_continuation','cd4','art1','art2','art3','bmi2','weight2',
                         'year_tbdx_cat.factor','region')){
  replacercs <- data.frame(var=c('age','female','bmi1','weight1','smearpos','culturepos','baccon','extrapulm',
                              'dot_continuation','cd4','art1','art2','art3','bmi2','weight2','year_tbdx_cat.factor'),
                        rcs=c('rcs(age,4)','female','rcs(bmi1,4)','rcs(weight1,4)','smearpos','culturepos','baccon','extrapulm',
                              'dot_continuation','rcs(cd4,4)','art1','art2','art3','rcs(bmi2,4)','rcs(weight2,4)','year_tbdx_cat.factor'))

# getpca <- function(var=c('age','female','bmi1','weight1','smearpos','culturepos','baccon','extrapulm','dot.partial','dot.complete','cd4','art1','art2','art3','bmi2','weight2',
#                          'X2...Cohort.or.Center.ID','year_tbdx_cat.factor')){
#   replacercs <- data.frame(var=c('age','female','bmi1','weight1','smearpos','culturepos','baccon','extrapulm','dot.partial',
#                               'dot.complete','cd4','art1','art2','art3','bmi2','weight2','X2...Cohort.or.Center.ID','year_tbdx_cat.factor'),
#                         rcs=c('rcs(age,4)','female','rcs(bmi1,4)','rcs(weight1,4)','smearpos','culturepos','baccon','extrapulm','dot.partial',
#                               'dot.complete','rcs(cd4,4)','art1','art2','art3','rcs(bmi2,4)','rcs(weight2,4)','X2...Cohort.or.Center.ID','year_tbdx_cat.factor'))

  prin_comp <- prcomp(idata[var], scale. = T)
  idata$pc1 <- prin_comp$x[,1]

  ## this does steps 2-5 in frank's book on page 82
  getpc1 <- function(df){
    newxs <- NULL

    for(i in 1:length(var)){
      f1 <- paste0("pc1~",paste(replacercs$rcs[match(var[i],replacercs$var)],collapse="+"))
      newx <- predict(ols(formula(f1),data=df))
      newxs <- cbind(newxs,newx)
    }
    prin_comp1 <- prcomp(newxs, scale. = T)
    prop_varex1 <- prin_comp1$sdev^2/sum(prin_comp1$sdev^2)
    return(list(pvar=sum(prop_varex1[1:2]),prin_comp1=prin_comp1,df=newxs))
  }

  dd <<- datadist(idata)
  options(datadist='dd')
  t1 <- getpc1(df=idata)
  idf <- as.data.frame(t1$df)
  names(idf) <- var
  idf$pc1 <- t1$prin_comp1$x[,1]

  t2 <- getpc1(idf)
  idf <- as.data.frame(t2$df)
  names(idf) <- var
  idf$pc1 <- t2$prin_comp1$x[,1]

  t3 <- getpc1(idf)
  idf <- as.data.frame(t3$df)
  names(idf) <- var
  idf$pc1 <- t3$prin_comp1$x[,1]

  t4 <- getpc1(idf)
  ## frank's book says to stop when the proportion of variation explained reaches a plateau which is typically 3-4 iterations, so we just do 4.
  prin_comp <- t4$prin_comp1

  #compute standard deviation of each principal component
  std_dev <- prin_comp$sdev
  #compute variance
  pr_var <- std_dev^2
  #check variance of first 10 components
  pr_var[1:10]
  #proportion of variance explained
  prop_varex <- pr_var/sum(pr_var)
  # number of components required to explain 80% of variance
  ncomp <- min(which(cumsum(prop_varex)>0.8))
  #scree plot
  # plot(prop_varex, xlab = "Principal Component",
  #      ylab = "Proportion of Variance Explained",
  #      type = "b")
  #cumulative scree plot
  # plot(cumsum(prop_varex), xlab = "Principal Component",
  #      ylab = "Cumulative Proportion of Variance Explained",
  #      type = "b")
  return(list(x=data.frame(patient=idata$patient,prin_comp$x[,1:min(ncomp,2)]),
              varprop=sum(prop_varex[1:min(ncomp,2)]),
              ncomp=ncomp,
              var=var))
}
getlrmsumline <- function(label,lrmsum,lrmsum2,pval,varprop,ncomp){
  if(length(lrmsum)>8){
    line <- paste0(sapply(lrmsum[,4],myformat2)," (",sapply(lrmsum[,6],myformat2),", ",sapply(lrmsum[,7],myformat2),")")
    line2 <- paste0(sapply(lrmsum2[,4],myformat2)," (",sapply(lrmsum2[,6],myformat2),", ",sapply(lrmsum2[,7],myformat2),")")
  }
  if(length(lrmsum)==8){
    line <- paste0(sapply(lrmsum[4],myformat2)," (",sapply(lrmsum[6],myformat2),", ",sapply(lrmsum[7],myformat2),")")
    line2 <- paste0(sapply(lrmsum2[4],myformat2)," (",sapply(lrmsum2[6],myformat2),", ",sapply(lrmsum2[7],myformat2),")")
  }
  if(length(line)>1) linetex <- paste(label,line,line2,c(myformat4(pval),rep("",length(line)-1)),
                                      c(paste0(round(100*varprop),"\\%"),rep("",length(line)-1)),sep=" & ") ## took out c(ncomp,rep("",length(line)-1))
  if(length(line)==1) linetex <- paste(label,line,line2,myformat4(pval),paste0(round(100*varprop),"\\%"),sep=" & ") ## took out ncomp
  return(linetex)
}

if(art_lvls == 4){
# Note:  Removed any site or region variable from before year_tbdx_cat in below and changed dot.partial/dot.complete to dot_intensive/dot_continuation in #2:

   age1 <- getpca(var=c('female','bmi1','smearpos','culturepos','baccon','extrapulm','cd4','art1','year_tbdx_cat.factor','region'))
   age2 <- getpca(var=c('female','bmi1','smearpos','culturepos','baccon','extrapulm','cd4','art1','dot_continuation','art2','art3','bmi2','year_tbdx_cat.factor','region'))

   sex1 <- getpca(var=c('age','bmi1','smearpos','culturepos','baccon','extrapulm','cd4','art1','year_tbdx_cat.factor','region'))
   sex2 <- getpca(var=c('age','bmi1','smearpos','culturepos','baccon','extrapulm','cd4','art1','dotintensive','dot_continuation','art2','art3','bmi2','year_tbdx_cat.factor','region'))

   ## use this for BMI and Weight 
   bmiwt1 <- getpca(var=c('age','female','smearpos','culturepos','baccon','extrapulm','cd4','art1','year_tbdx_cat.factor','region'))
   bmiwt2 <- getpca(var=c('age','female','smearpos','culturepos','baccon','extrapulm','cd4','art1','dot_continuation','art2','art3','year_tbdx_cat.factor','region'))

   ## use this for smear, culture, and bacteriologically confirmed
   test1 <- getpca(var=c('age','female','bmi1','extrapulm','cd4','art1','year_tbdx_cat.factor','region'))
   test2 <- getpca(var=c('age','female','bmi1','extrapulm','cd4','art1','dot_continuation','art2','art3','bmi2','year_tbdx_cat.factor','region'))

   extra1 <- getpca(var=c('age','female','bmi1','smearpos','culturepos','baccon','cd4','art1','year_tbdx_cat.factor','region'))
   extra2 <- getpca(var=c('age','female','bmi1','smearpos','culturepos','baccon','cd4','art1','dot_continuation','art2','art3','bmi2','year_tbdx_cat.factor','region'))

   cd41 <- getpca(var=c('age','female','bmi1','smearpos','culturepos','baccon','extrapulm','art1','year_tbdx_cat.factor','region'))
   cd42 <- getpca(var=c('age','female','bmi1','smearpos','culturepos','baccon','extrapulm','art1','dot_continuation','art2','art3','bmi2','year_tbdx_cat.factor','region'))

   ## use this for ART at diagnosis and ART timing
   art1 <- getpca(var=c('age','female','bmi1','smearpos','culturepos','baccon','extrapulm','cd4','year_tbdx_cat.factor','region'))
   art2 <- getpca(var=c('age','female','bmi1','smearpos','culturepos','baccon','extrapulm','cd4','dot_continuation','bmi2','year_tbdx_cat.factor','region'))

   dot2 <- getpca(var=c('age','female','bmi1','smearpos','culturepos','baccon','extrapulm','cd4','art1','art2','art3','bmi2','year_tbdx_cat.factor','region'))


   library(rms)
   df1$dot <- ifelse(df1$dot.partial==1,2,ifelse(df1$dot.complete,3,1))
   df1$dot <- factor(df1$dot,1:3,c("None","Partial","Complete"))
   # df1$art <- ifelse(df1$art1==1,1,ifelse(df1$art2,2,ifelse(df1$art3==1,3,4)))
   # df1$art <- factor(df1$art,1:4,c("On ART","Initiated within 8 weeks","Initiated after 8 weeks","No ART during TB treatment"))
   df1$art <- ifelse(df1$art1==1,3,ifelse(df1$art2,1,2))
   df1$art <- factor(df1$art,1:3,c(paste("Initiated within ",post_tbdx_time/7," weeks",sep=''),
                                   paste("Initiated after ",post_tbdx_time/7," weeks or never",sep=''),
                                   "Already on ART"))

   df1$bmidelta <- df1$bmi2-df1$bmi1
   df1$weightdelta <- df1$weight2-df1$weight1

   #moddata <- merge(merge(df1,age1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,age1$x),d1[c('patient',new_out_short)])

   dd <- datadist(moddata)
   options(datadist='dd')

   ## AGE
   #moddata <- merge(merge(df1,age1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,age1$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ rcs(age,4) + PC1 + PC2',sep=''))
   fmla1 <- as.formula(paste(new_out_short,'~ age + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ age',sep=''))

   #lrm1var1.nl <- lrm(recfail~rcs(age,4)+PC1+PC2,moddata)
   #lrm1var1 <- lrm(recfail~age+PC1+PC2,moddata)
   #ulrm1var1 <- lrm(recfail~age,moddata)
   lrm1var1.nl <- lrm(fmla,moddata)
   lrm1var1 <- lrm(fmla1,moddata)
   ulrm1var1 <- lrm(fmlau,moddata)

   lrm1line1 <- getlrmsumline(label="Age (per 10 years)",
                              lrmsum=summary(ulrm1var1,age=c(30,40))[2,],
                              lrmsum2=summary(lrm1var1,age=c(30,40))[2,],
                              pval=as.data.frame(anova(lrm1var1))$P[1],
                              varprop=age1$varprop,ncomp=age1$ncomp)

   #moddata <- merge(merge(df1,age2$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,age2$x),d1[c('patient',new_out_short)])

   #lrm2var1.nl <- lrm(recfail~rcs(age,4)+PC1+PC2,moddata)
   #lrm2var1 <- lrm(recfail~age+PC1+PC2,moddata)
   #ulrm2var1 <- lrm(recfail~age,moddata)
   lrm2var1.n1 <- lrm(fmla,moddata)
   lrm2var1 <- lrm(fmla1,moddata)
   ulrm2var1 <- lrm(fmlau,moddata)

   lrm2line1 <- getlrmsumline(label="Age (per 10 years)",
                              lrmsum=summary(ulrm2var1,age=c(30,40))[2,],
                              lrmsum2=summary(lrm2var1,age=c(30,40))[2,],
                              pval=as.data.frame(anova(lrm2var1))$P[1],
                              varprop=age2$varprop,ncomp=age2$ncomp)

   ## SEX
   #moddata <- merge(merge(df1,sex1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,sex1$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ female + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ female',sep=''))

   #lrm1var2 <- lrm(recfail~female+PC1+PC2,moddata)
   #ulrm1var2 <- lrm(recfail~female,moddata)
   lrm1var2 <- lrm(fmla, moddata)
   ulrm1var2 <- lrm(fmlau, moddata)

   lrm1line2 <- getlrmsumline(label="Female vs. Male",
                              lrmsum=summary(ulrm1var2)[2,],
                              lrmsum2=summary(lrm1var2)[2,],
                              pval=as.data.frame(anova(lrm1var2))$P[1],
                              varprop=sex1$varprop,ncomp=sex1$ncomp)

   moddata <- merge(merge(df1,sex2$x),d1[c("patient","recfail")])
   lrm2var2 <- lrm(recfail~female+PC1+PC2,moddata)
   ulrm2var2 <- lrm(recfail~female,moddata)

   lrm2line2 <- getlrmsumline(label="Female vs. Male",
                              lrmsum=summary(ulrm2var2)[2,],
                              lrmsum2=summary(lrm2var2)[2,],
                              pval=as.data.frame(anova(lrm2var2))$P[1],
                              varprop=sex2$varprop,ncomp=sex2$ncomp)

   ## BMI
   #moddata <- merge(merge(df1,bmiwt1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,bmiwt1$x), d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ rcs(bmi1,4) + PC1 + PC2',sep=''))
   fmla1 <- as.formula(paste(new_out_short,'~ bmi1 + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ bmi1',sep=''))

   #lrm1var3.nl <- lrm(recfail~rcs(bmi1,4)+PC1+PC2,moddata)
   #lrm1var3 <- lrm(recfail~bmi1+PC1+PC2,moddata)
   #ulrm1var3 <- lrm(recfail~bmi1,moddata)
   lrm1var3.n1 <- lrm(fmla, moddata)
   lrm1var3 <- lrm(fmla1, moddata)
   ulrm1var3 <- lrm(fmlau, moddata)

   lrm1line3 <- getlrmsumline(label="BMI at start of TB treatment (per 1 unit)",
                              lrmsum=summary(ulrm1var3,bmi1=c(20,21))[2,],
                              lrmsum2=summary(lrm1var3,bmi1=c(20,21))[2,],
                              pval=as.data.frame(anova(lrm1var3))$P[1],
                              varprop=bmiwt1$varprop,ncomp=bmiwt1$ncomp)

   #moddata <- merge(merge(df1,bmiwt2$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,bmiwt2$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ rcs(bmidelta,4) + rcs(bmi1,4) + PC1 + PC2',sep=''))
   fmla1 <- as.formula(paste(new_out_short,'~ bmidelta + bmi1 + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ bmidelta',sep=''))

   #lrm2var3.nl <- lrm(recfail~rcs(bmidelta,4)+rcs(bmi1,4)+PC1+PC2,moddata)
   #lrm2var3 <- lrm(recfail~bmidelta+bmi1+PC1+PC2,moddata)
   #ulrm2var3 <- lrm(recfail~bmidelta,moddata)
   lrm2var3.n1 <- lrm(fmla, moddata)
   lrm2var3 <- lrm(fmla1, moddata)
   ulrm2var3 <- lrm(fmlau, moddata)

   lrm2line3 <- getlrmsumline(label="BMI change (per 2 unit increase)",
                              lrmsum=summary(ulrm2var3,bmidelta=c(0,2))[2,],
                              lrmsum2=summary(lrm2var3,bmidelta=c(0,2))[2,],
                              pval=as.data.frame(anova(lrm2var3))$P[1],
                              varprop=bmiwt2$varprop,ncomp=bmiwt2$ncomp)

   ## WEIGHT
   dd <- datadist(moddata)
   options(datadist='dd')

   #moddata <- merge(merge(df1,bmiwt1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,bmiwt1$x), d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ rcs(weight1,4) + PC1 + PC2',sep=''))
   fmla1 <- as.formula(paste(new_out_short,'~ weight1 + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ weight1',sep=''))

   #lrm1var4.nl <- lrm(recfail~rcs(weight1,4)+PC1+PC2,moddata)
   #lrm1var4 <- lrm(recfail~weight1+PC1+PC2,moddata)
   #ulrm1var4 <- lrm(recfail~weight1,moddata)

   lrm1var4.n1 <- lrm(fmla, moddata)
   lrm1var4 <- lrm(fmla1, moddata)
   ulrm1var4 <- lrm(fmlau, moddata)

   lrm1line4 <- getlrmsumline(label="Weight at start of TB treatment (per 1 kg)",
                              lrmsum=summary(ulrm1var4,weight1=c(30,31))[2,],
                              lrmsum2=summary(lrm1var4,weight1=c(30,31))[2,],
                              pval=as.data.frame(anova(lrm1var4))$P[1],
                              varprop=bmiwt1$varprop,ncomp=bmiwt1$ncomp)

   #moddata <- merge(merge(df1,bmiwt2$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,bmiwt2$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ rcs(weightdelta,4) + rcs(weight1,4) + PC1 + PC2',sep=''))
   fmla1 <- as.formula(paste(new_out_short,'~ weightdelta + weight1 + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short, '~ weightdelta',sep=''))

   #lrm2var4.nl <- lrm(recfail~rcs(weightdelta,4)+rcs(weight1,4)+PC1+PC2,moddata)
   #lrm2var4 <- lrm(recfail~weightdelta+weight1+PC1+PC2,moddata)
   #ulrm2var4 <- lrm(recfail~weightdelta,moddata)
   lrm2var4.n1 <- lrm(fmla, moddata)
   lrm2var4 <- lrm(fmla1, moddata)
   ulrm2var4 <- lrm(fmlau, moddata)

   lrm2line4 <- getlrmsumline(label="Weight change (per 5 kg increase)",
                              lrmsum=summary(ulrm2var4,weightdelta=c(0,5))[2,],
                              lrmsum2=summary(lrm2var4,weightdelta=c(0,5))[2,],
                              pval=as.data.frame(anova(lrm2var4))$P[1],
                              varprop=bmiwt2$varprop,ncomp=bmiwt2$ncomp)

   ## SMEAR
   #moddata <- merge(merge(df1,test1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,test1$x), d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ smearpos + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ smearpos',sep=''))

   #lrm1var5 <- lrm(recfail~smearpos+PC1+PC2,moddata)
   #ulrm1var5 <- lrm(recfail~smearpos,moddata)
   lrm1var5 <- lrm(fmla, moddata)
   ulrm1var5 <- lrm(fmlau, moddata)

   lrm1line5 <- getlrmsumline(label="Smear Positive",
                              lrmsum=summary(ulrm1var5)[2,],
                              lrmsum2=summary(lrm1var5)[2,],
                              pval=as.data.frame(anova(lrm1var5))$P[1],
                              varprop=test1$varprop,ncomp=test1$ncomp)

   #moddata <- merge(merge(df1,test2$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1, test2$x), d1[c('patient',new_out_short)])

   #lrm2var5 <- lrm(recfail~smearpos+PC1+PC2,moddata)
   #ulrm2var5 <- lrm(recfail~smearpos,moddata)
   lrm2var5 <- lrm(fmla, moddata)
   ulrm2var5 <- lrm(fmlau, moddata)

   lrm2line5 <- getlrmsumline(label="Smear Positive",
                              lrmsum=summary(ulrm2var5)[2,],
                              lrmsum2=summary(lrm2var5)[2,],
                              pval=as.data.frame(anova(lrm2var5))$P[1],
                              varprop=test2$varprop,ncomp=test2$ncomp)

   ## CULTURE
   #moddata <- merge(merge(df1,test1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,test1$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short, '~ culturepos + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short, '~ culturepos',sep=''))

   #lrm1var6 <- lrm(recfail~culturepos+PC1+PC2,moddata)
   #ulrm1var6 <- lrm(recfail~culturepos,moddata)
   lrm1var6 <- lrm(fmla, moddata)
   ulrm1var6 <- lrm(fmlau, moddata)

   lrm1line6 <- getlrmsumline(label="Culture Positive",
                              lrmsum=summary(ulrm1var6)[2,],
                              lrmsum2=summary(lrm1var6)[2,],
                              pval=as.data.frame(anova(lrm1var6))$P[1],
                              varprop=test1$varprop,ncomp=test1$ncomp)

   #moddata <- merge(merge(df1,test2$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1, test2$x),d1[c('patient',new_out_short)])

   #lrm2var6 <- lrm(recfail~culturepos+PC1+PC2,moddata)
   #ulrm2var6 <- lrm(recfail~culturepos,moddata)
   lrm2var6 <- lrm(fmla, moddata)
   ulrm2var6 <- lrm(fmlau, moddata)

   lrm2line6 <- getlrmsumline(label="Culture Positive",
                              lrmsum=summary(ulrm2var6)[2,],
                              lrmsum2=summary(lrm2var6)[2,],
                              pval=as.data.frame(anova(lrm2var6))$P[1],
                              varprop=test2$varprop,ncomp=test2$ncomp)
   ## BAC POS
   #moddata <- merge(merge(df1,test1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1, test1$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ baccon + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~baccon',sep=''))

   #lrm1var7 <- lrm(recfail~baccon+PC1+PC2,moddata)
   #ulrm1var7 <- lrm(recfail~baccon,moddata)
   lrm1var7 <- lrm(fmla, moddata)
   ulrm1var7 <- lrm(fmlau, moddata)

   lrm1line7 <- getlrmsumline(label="Bacteriologically Confirmed",
                              lrmsum=summary(ulrm1var7)[2,],
                              lrmsum2=summary(lrm1var7)[2,],
                              pval=as.data.frame(anova(lrm1var7))$P[1],
                              varprop=test1$varprop,ncomp=test1$ncomp)

   #moddata <- merge(merge(df1,test2$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,test2$x),d1[c('patient',new_out_short)])

   #lrm2var7 <- lrm(recfail~baccon+PC1+PC2,moddata)
   #ulrm2var7 <- lrm(recfail~baccon,moddata)
   lrm2var7 <- lrm(fmla, moddata)
   ulrm2var7 <- lrm(fmlau, moddata)

   lrm2line7 <- getlrmsumline(label="Bacteriologically Confirmed",
                              lrmsum=summary(ulrm2var7)[2,],
                              lrmsum2=summary(lrm2var7)[2,],
                              pval=as.data.frame(anova(lrm2var7))$P[1],
                              varprop=test2$varprop,ncomp=test2$ncomp)
   ## EXTRAPULMONARY
   #moddata <- merge(merge(df1,extra1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,extra1$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ extrapulm + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ extrapulm',sep=''))

   #lrm1var8 <- lrm(recfail~extrapulm+PC1+PC2,moddata)
   #ulrm1var8 <- lrm(recfail~extrapulm,moddata)
   lrm1var8 <- lrm(fmla, moddata)
   ulrm1var8 <- lrm(fmlau, moddata)

   lrm1line8 <- getlrmsumline(label="Extrapulmonary vs. pulmonary/both/unknown",
                              lrmsum=summary(ulrm1var8)[2,],
                              lrmsum2=summary(lrm1var8)[2,],
                              pval=as.data.frame(anova(lrm1var8))$P[1],
                              varprop=extra1$varprop,ncomp=extra1$ncomp)

   #moddata <- merge(merge(df1,extra2$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,extra2$x),d1[c('patient',new_out_short)])

   #lrm2var8 <- lrm(recfail~extrapulm+PC1+PC2,moddata)
   #ulrm2var8 <- lrm(recfail~extrapulm,moddata)
   lrm2var8 <- lrm(fmla, moddata)
   ulrm2var8 <- lrm(fmlau, moddata)

   lrm2line8 <- getlrmsumline(label="Extrapulmonary vs. pulmonary/both/unknown",
                              lrmsum=summary(ulrm2var8)[2,],
                              lrmsum2=summary(lrm2var8)[2,],
                              pval=as.data.frame(anova(lrm2var8))$P[1],
                              varprop=extra2$varprop,ncomp=extra2$ncomp)
   ## cd4
   #moddata <- merge(merge(df1,cd41$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,cd41$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ rcs(cd4,4) + PC1 + PC2',sep=''))
   fmla1 <- as.formula(paste(new_out_short,'~ cd4 + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ cd4',sep=''))

   #lrm1var9.nl <- lrm(recfail~rcs(cd4,4)+PC1+PC2,moddata)
   #lrm1var9 <- lrm(recfail~cd4+PC1+PC2,moddata)
   #ulrm1var9 <- lrm(recfail~cd4,moddata)
   lrm1var9.n1 <- lrm(fmla, moddata)
   lrm1var9 <- lrm(fmla1, moddata)
   ulrm1var9 <- lrm(fmlau, moddata)

   lrm1line9 <- getlrmsumline(label="CD4+ cell count (per 100 cells)",
                              lrmsum=summary(ulrm1var9,cd4=c(0,100))[2,],
                              lrmsum2=summary(lrm1var9,cd4=c(0,100))[2,],
                              pval=as.data.frame(anova(lrm1var9))$P[1],
                              varprop=cd41$varprop,ncomp=cd41$ncomp)

   #moddata <- merge(merge(df1,cd42$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,cd42$x),d1[c('patient',new_out_short)])

   #lrm2var9.nl <- lrm(recfail~rcs(cd4,4)+PC1+PC2,moddata)
   #lrm2var9 <- lrm(recfail~cd4+PC1+PC2,moddata)
   #ulrm2var9 <- lrm(recfail~cd4,moddata)
   lrm2var9.n1 <- lrm(fmla, moddata)
   lrm2var9 <- lrm(fmla1, moddata)
   ulrm2var9 <- lrm(fmlau, moddata)

   lrm2line9 <- getlrmsumline(label="CD4+ cell count (per 100 cells)",
                              lrmsum=summary(ulrm2var9,cd4=c(0,100))[2,],
                              lrmsum2=summary(lrm2var9,cd4=c(0,100))[2,],
                              pval=as.data.frame(anova(lrm2var9))$P[1],
                              varprop=cd42$varprop,ncomp=cd42$ncomp)
   ## ART
   #moddata <- merge(merge(df1,art1$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,art1$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ art1 + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ art1',sep=''))

   #lrm1var10 <- lrm(recfail~art1+PC1+PC2,moddata)
   #ulrm1var10 <- lrm(recfail~art1,moddata)
   lrm1var10 <- lrm(fmla, moddata)
   ulrm1var10 <- lrm(fmlau, moddata)

   lrm1line10 <- getlrmsumline(label="On ART",
                               lrmsum=summary(ulrm1var10)[2,],
                               lrmsum2=summary(lrm1var10)[2,],
                               pval=as.data.frame(anova(lrm1var10))$P[1],
                               varprop=art1$varprop,ncomp=art1$ncomp)

   #moddata <- merge(merge(df1,art2$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,art2$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ art + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ art',sep=''))

   #lrm2var10 <- lrm(recfail~art+PC1+PC2,moddata)
   #ulrm2var10 <- lrm(recfail~art,moddata)
   lrm2var10 <- lrm(fmla, moddata)
   ulrm2var10 <- lrm(fmlau, moddata)

   lrm2line10 <- getlrmsumline(label=c(paste('ART after ',post_tbdx_time/7,' weeks or never vs. ART in ',post_tbdx_time/7,' weeks',sep=''),
                                       paste('Already on ART vs. ART in ',post_tbdx_time/7,' weeks',sep='')),
                               lrmsum=summary(ulrm2var10,art=paste("Initiated within ",post_tbdx_time/7," weeks",sep=''))[c(2,4),],
                               lrmsum2=summary(lrm2var10,art=paste("Initiated within ",post_tbdx_time/7," weeks",sep=''))[c(6,8),],
                               pval=as.data.frame(anova(lrm2var10))$P[1],
                               varprop=art2$varprop,ncomp=art2$ncomp)

   ## DOT
   #moddata <- merge(merge(df1,dot2$x),d1[c("patient","recfail")])
   moddata <- merge(merge(df1,dot2$x),d1[c('patient',new_out_short)])

   fmla <- as.formula(paste(new_out_short,'~ dot + PC1 + PC2',sep=''))
   fmlau <- as.formula(paste(new_out_short,'~ dot',sep=''))

   #lrm2var11 <- lrm(recfail~dot+PC1+PC2,moddata)
   #ulrm2var11 <- lrm(recfail~dot,moddata)
   lrm2var11 <- lrm(fmla, moddata)
   ulrm2var11 <- lrm(fmlau, moddata)

   lrm2line11 <- getlrmsumline(label=c("Partial DOT vs. None","Complete DOT vs. None"),
                               lrmsum=summary(ulrm2var11,dot="None")[c(2,4),],
                               lrmsum2=summary(lrm2var11,dot="None")[c(6,8),],
                               pval=as.data.frame(anova(lrm2var11))$P[1],
                               varprop=dot2$varprop,ncomp=dot2$ncomp)

   lrm1table <- paste(unlist(sapply(paste0("lrm1line",1:10),get)),collapse=" \\\\ ")
   lrm2table <- paste(unlist(sapply(paste0("lrm2line",1:11),get)),collapse=" \\\\ ")


}

